/**
 * @fileOverview
 * @name e1.js
 * @author Wagner Müller Gegler, Roberto Oppermann Cordoni
 */

/**
 * @typedef {Object} E1Controller~E1InterfaceParams - Configuration parameters for an e1 interface.
 * @property {String} clock - [enabled|disabled] Clock Recovery.
 * @property {String} crc - [enabled|disabled] Enable CRC4 checking.
 * @property {String} enabled - [enabled|disabled] Determines if e1 span is enabled.
 * @property {String} fractional - [enabled|disabled] Set fractional E1.
 * @property {String} mfcr2_allow_collect_calls - [yes|no] Allow collect calls.
 * @property {String} mfcr2_double_answer - [yes|no] Enable double-answer process to block collect calls.
 * @property {String} mfcr2_get_ani_first - [yes|no] Get the ANI before getting DNIS.
 * @property {Number} mfcr2_max_ani - Maximum amount of ANI digits.
 * @property {Number} mfcr2_max_dnis - Maximum amount of DNIS digits.
 * @property {Number} numchans - Number of channels to use in case of fractional="enabled".
 * @property {String} overlapdial - [yes|no] Overlap dialing mode.
 * @property {String} signalling - [mfcr2|pri_net|pri_cpe] Signalling method.
 * @property {String} switchtype - [euroisdn|national|qsig] Specify the switchtype.
 */

/**
 * @class E1Controller
 * @param {Object} $http - AngularJS service for HTTP communication.
 * @param {Object} $log - AngularJS service for logging.
 * @param {Object} $anchorScroll - AngularJS service for scroll page.
 * @property {NetworkController~E1InterfaceParams[]} interfaces - List of each e1 interface parameters.
 * @property {String[]} textfile - List of plaintexts to be saved in a file. textfile[0] = system.conf and textfile[1] = chan_dahdi.conf.
 */

angular.module('ag561app').controller('E1Controller', ['$http', '$log', '$anchorScroll', '$location',
                                   function($http, $log, $anchorScroll, $location) {
  var url_tdm = '/api/config/tdm';
  var ctrl = this;
  ctrl.loading = false;
  ctrl.invalid_config = 0;

  /**
   * Clear alert message from web page, but do this only
   * if we aren't with some error. We just want to erase
   * the message of danger if all the variables are correct.
   *
   * @function clear_alert_messages
   * @memberof NetworkController
   */
  ctrl.clear_alert_messages = function() {
    if (!ctrl.invalid_config) {
      ctrl.alert_danger = false;
      ctrl.alert_success = false;
    }
  };

  /**
   * Get parameters of e1 interfaces.
   * @function get
   * @memberof E1Controller
   */
  ctrl.get = function() {
    $http({
      method: 'GET',
      url: url_tdm
      }).then(function(response) {
        // First function handles success
        ctrl.ports = {};
        ctrl.ports = response.data.ports;
        ctrl.mfcr2_tone_amp = response.data.mfcr2_tone_amp;

        // Show first port by default
        ctrl.ports[0].edit = true;
      }, function(response) {
        // Second function handles error
        ctrl.error = response.data;
      });
  };

  /**
   * Save the new configuration.
   * @function save
   * @memberof E1Controller
   */
  ctrl.save = function() {
    ctrl.loading = true;

    // Remove variables that cannot go to API
    for (var i = 0; i < ctrl.ports.length; i++) {
      delete ctrl.ports[i].edit;
      delete ctrl.ports[i].invalid_mfcr2_max_ani;
      delete ctrl.ports[i].invalid_mfcr2_max_dnis;
      delete ctrl.ports[i].invalid_keepalive_host;
    }

    var data = {
      'ports' : ctrl.ports,
      'mfcr2_tone_amp' : ctrl.mfcr2_tone_amp
    }
    $http({
      method: 'PUT',
      url: url_tdm,
      data: data,
      }).then(function(response) {
        // First function handles success
        ctrl.loading = false;
        ctrl.alert_success = true;
        ctrl.get();
      }, function(response) {
        // Second function handles error
        ctrl.loading = false;
        ctrl.alert_danger = true;
      });
  };

  /**
   * Check if some configuration is invalid. If true, then disable the
   * button to 'save' config and show the 'danger message'.
   *
   * @function __check_invalid_config
   * @memberof E1Controller
   */
  __check_invalid_config = function() {
    /* Check E1 port config */
    for (var i = 0; i < ctrl.ports.length; i++) {
        var port = ctrl.ports[i];
        if (port.invalid_mfcr2_max_ani || port.invalid_mfcr2_max_dnis || port.invalid_keepalive_host) {
            ctrl.invalid_config = true;
            ctrl.alert_danger = true;
            return;
        }
    }
        
    ctrl.invalid_config = false;
    ctrl.alert_danger = false;
  };

  ctrl.validateMfcr2_max_ani = function(port) {
    port.invalid_mfcr2_max_ani = false;
    if (port.signalling == 'mfcr2')
      port.invalid_mfcr2_max_ani = !(port.mfcr2_max_ani >= 0 && port.mfcr2_max_ani < 64);

    __check_invalid_config();
  };

  ctrl.validateMfcr2_max_dnis = function(port) {
    port.invalid_mfcr2_max_ani = false;
    if (port.signalling == 'mfcr2')
      port.invalid_mfcr2_max_dnis = !(port.mfcr2_max_dnis > 0 && port.mfcr2_max_dnis < 64);

    __check_invalid_config();
  };

  ctrl.validateKeepaliveHost = function(port) {
    var pattern = /^([\w.-]+)(:[\d]+)?$/;
    port.invalid_keepalive_host = port.keepalive_host && !port.keepalive_host.match(pattern);

    __check_invalid_config();
  };

  ctrl.get();

}]);
